/* Copyright © 2013 Matthew Champion
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
 * Neither the name of mattunderscore.com nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL MATTHEW CHAMPION BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE. */

package com.mattunderscore.filter.contentnegotiation.variantsource;

import java.io.File;
import java.io.InputStream;
import java.net.URL;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import com.mattunderscore.filter.contentnegotiation.parser.Variant;

/**
 * A variant source that parses XML for variants. It uses SAX to parse the XML. The XML is passed in
 * through an input stream. A SAX handler to use can also be passed in. By default it uses the
 * {@link VariantXMLParser} class.
 * 
 * @author Matt Champion
 * @since 0.2.0
 */
public class BaseVariantXMLSource implements InputStreamVariantSource
{
    private static final String JAXP_SCHEMA_LANGUAGE = "http://java.sun.com/xml/jaxp/properties/schemaLanguage";
    private static final String W3C_XML_SCHEMA = "http://www.w3.org/2001/XMLSchema"; 
    private static final String JAXP_SCHEMA_SOURCE = "http://java.sun.com/xml/jaxp/properties/schemaSource";
    private static final SAXParserFactory factory = SAXParserFactory.newInstance();
    private static final Logger log = Logger.getLogger(FilterVariantXMLSource.class.getCanonicalName());

    private InputStream inputStream;
    private VariantSAXHandler parser;
    
    static {
        factory.setValidating(true);
    }

    public BaseVariantXMLSource()
    {
        this(new VariantXMLParser());
    }
    
    public BaseVariantXMLSource(final VariantSAXHandler parser)
    {
        setHandler(parser);
    }
    
    /**
     * Setter for the SAX handler.
     * @param handler The SAX handler
     * @since 0.2.0
     */
    public void setHandler(final VariantSAXHandler handler)
    {
        this.parser = handler;
    }

    @Override
    public List<Variant> getVariants() throws VariantSourceConfigurationException, VariantSourceException
    {
        if (inputStream == null)
        {
            throw new VariantSourceConfigurationException("No input stream has been set");
        }
        else
        {
            try
            {
                SAXParser saxParser = factory.newSAXParser();
                URL negotiationUrl = BaseVariantXMLSource.class.getResource("/negotiation.xsd");
                saxParser.setProperty(JAXP_SCHEMA_LANGUAGE, W3C_XML_SCHEMA);
                saxParser.setProperty(JAXP_SCHEMA_SOURCE, new File(negotiationUrl.getFile()));
                saxParser.parse(inputStream, parser);
                return parser.getVariants();
            }
            catch (Throwable throwable)
            {
                log.log(Level.WARNING, "BaseVariantXMLSource unable to return variants.", throwable);
                throw new VariantSourceException("BaseVariantXMLSource unable to return variants.",
                        throwable);
            }
        }
    }

    @Override
    public void setInputStream(final InputStream inputStream)
    {
        this.inputStream = inputStream;
    }
}
